@@ -10,7 +10,7 @@ os.environ['LANGUAGE'] = 'C'
 
 sys.path.insert(0, '..')
 from zeroinstall.injector import qdom
-from zeroinstall.injector import iface_cache, download, distro
+from zeroinstall.injector import iface_cache, download, distro, model
 from zeroinstall.zerostore import Store; Store._add_with_helper = lambda *unused: False
 from zeroinstall import support, helpers
 from zeroinstall.support import basedir
@@ -22,6 +22,14 @@ empty_feed = qdom.parse(StringIO.StringIO("""<interface xmlns='http://zero-insta
 <summary>just for testing</summary>
 </interface>"""))
 
+import my_dbus
+sys.modules['dbus'] = my_dbus
+sys.modules['dbus.glib'] = my_dbus
+my_dbus.types = my_dbus
+sys.modules['dbus.types'] = my_dbus
+sys.modules['dbus.mainloop'] = my_dbus
+sys.modules['dbus.mainloop.glib'] = my_dbus
+
 mydir = os.path.dirname(__file__)
 
 # Catch us trying to run the GUI and return a dummy string instead
@@ -33,6 +41,20 @@ def test_execvp(prog, args):
 
 os.execvp = test_execvp
 
+test_locale = (None, None)
+assert model.locale
+class TestLocale:
+	LC_ALL = 0	# Note: LC_MESSAGES not present on Windows
+	def getlocale(self, x):
+		return test_locale
+model.locale = TestLocale()
+
+class DummyPackageKit:
+	available = False
+
+	def get_candidates(self, package, factory, prefix):
+		pass
+
 class BaseTest(unittest.TestCase):
 	def setUp(self):
 		warnings.resetwarnings()
@@ -66,6 +88,9 @@ class BaseTest(unittest.TestCase):
 
 		distro._host_distribution = distro.DebianDistribution(dpkgdir + '/status',
 								      dpkgdir + '/pkgcache.bin')
+		distro._host_distribution._packagekit = DummyPackageKit()
+
+		my_dbus.system_services = {}
 	
 	def tearDown(self):
 		shutil.rmtree(self.config_home)
