@@ -28,10 +28,6 @@ def raise_gui(*args):
 	ran_gui = True
 background._detach = lambda: False
 background._exec_gui = raise_gui
-sys.modules['dbus'] = my_dbus
-sys.modules['dbus.glib'] = my_dbus
-my_dbus.types = my_dbus
-sys.modules['dbus.types'] = my_dbus
 
 @contextmanager
 def output_suppressed():
@@ -73,6 +69,10 @@ class DummyHandler(handler.Handler):
 		#import traceback
 		#traceback.print_exc()
 
+class NetworkManager:
+	def state(self):
+		return 3	# NM_STATUS_CONNECTED
+
 class TestDownload(BaseTest):
 	def setUp(self):
 		BaseTest.setUp(self)
@@ -208,6 +208,34 @@ class TestDownload(BaseTest):
 				if "HelloWorld/Missing" not in str(ex):
 					raise ex
 	
+	def testDistro(self):
+		with output_suppressed():
+			native_url = 'http://example.com:8000/Native.xml'
+
+			# Initially, we don't have the feed at all...
+			master_feed = iface_cache.iface_cache.get_feed(native_url)
+			assert master_feed is None, master_feed
+
+			trust.trust_db.trust_key('DE937DD411906ACF7C263B396FCF121BE2390E0B', 'example.com:8000')
+			self.child = server.handle_requests('Native.xml', '6FCF121BE2390E0B.gpg', '/key-info/key/DE937DD411906ACF7C263B396FCF121BE2390E0B')
+			h = DummyHandler()
+			policy = autopolicy.AutoPolicy(native_url, download_only = False, handler = h)
+			assert policy.need_download()
+
+			solve = policy.solve_with_downloads()
+			h.wait_for_blocker(solve)
+			tasks.check(solve)
+
+			master_feed = iface_cache.iface_cache.get_feed(native_url)
+			assert master_feed is not None
+			assert master_feed.implementations == {}
+
+			distro_feed_url = master_feed.get_distro_feed()
+			assert distro_feed_url is not None
+			distro_feed = iface_cache.iface_cache.get_feed(distro_feed_url)
+			assert distro_feed is not None
+			assert len(distro_feed.implementations) == 2, distro_feed.implementations
+
 	def testWrongSize(self):
 		with output_suppressed():
 			self.child = server.handle_requests('Hello-wrong-size', '6FCF121BE2390E0B.gpg',
@@ -357,6 +385,7 @@ class TestDownload(BaseTest):
 		try:
 			sys.stdout = StringIO()
 			self.child = server.handle_requests('Hello.xml', '6FCF121BE2390E0B.gpg')
+			my_dbus.system_services = {"org.freedesktop.NetworkManager": {"/org/freedesktop/NetworkManager": NetworkManager()}}
 			my_dbus.user_callback = choose_download
 			pid = os.getpid()
 			old_exit = os._exit
