@@ -7,12 +7,15 @@ sys.path.insert(0, '..')
 from zeroinstall.injector import autopolicy, reader, model
 from zeroinstall.injector.iface_cache import iface_cache
 
+import warnings
 import logging
 logger = logging.getLogger()
 #logger.setLevel(logging.DEBUG)
 
 class TestPolicy(BaseTest):
 	def testSource(self):
+		warnings.filterwarnings("ignore", category = DeprecationWarning)
+
 		foo = iface_cache.get_interface('http://foo/Binary.xml')
 		reader.update(foo, 'Binary.xml')
 		foo_src = iface_cache.get_interface('http://foo/Source.xml')
@@ -23,7 +26,7 @@ class TestPolicy(BaseTest):
 		p = autopolicy.AutoPolicy('http://foo/Binary.xml', dry_run = True)
 		p.freshness = 0
 		p.network_use = model.network_full
-		p.recalculate()
+		p.recalculate()	# Deprecated
 		assert p.implementation[foo].id == 'sha1=123'
 
 		# Now ask for source instead
