@@ -2,6 +2,7 @@
 # See the README file for details, or visit http://0install.net.
 
 import zeroinstall
+import os
 from zeroinstall.support import tasks
 from zeroinstall.injector.model import Interface, Feed, stable, testing, developer, stability_levels
 from zeroinstall.injector.iface_cache import iface_cache
@@ -138,7 +139,7 @@ class Description:
 					buffer.insert_with_tags(iter, _('Valid signature by "%(name)s"\n- Dated: %(sig_date)s\n- Fingerprint: %(sig_fingerprint)s\n') %
 							{'name': name, 'sig_date': time.strftime('%c', time.localtime(sig.get_timestamp())), 'sig_fingerprint': sig.fingerprint})
 					if not sig.is_trusted():
-						if feed.url.startswith('/'):
+						if os.path.isabs(feed.url):
 							buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list') + '\n')
 						else:
 							buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list (either you removed it, or '
@@ -180,7 +181,8 @@ class Feeds:
 					if x.user_override:
 						interface.extra_feeds.remove(x)
 						writer.save_interface(interface)
-						policy.recalculate()
+						import main
+						main.recalculate()
 						return
 					else:
 						dialog.alert(self.get_toplevel(),
@@ -281,16 +283,17 @@ class Properties:
 			i = 0
 		stability.set_active(i)
 
-		def set_stability_policy(combo):
+		def set_stability_policy(combo, stability = stability):	# (pygtk bug?)
 			i = stability.get_active()
 			if i == 0:
 				new_stability = None
 			else:
-				name = stability.get_model()[i][0].lower()
+				name = ['stable', 'testing', 'developer'][i-1]
 				new_stability = stability_levels[name]
 			interface.set_stability_policy(new_stability)
 			writer.save_interface(interface)
-			policy.recalculate()
+			import main
+			main.recalculate()
 		stability.connect('changed', set_stability_policy)
 
 		self.use_list = ImplementationList(policy, interface, widgets)
@@ -397,7 +400,8 @@ def add_remote_feed(policy, parent, interface):
 							interface.extra_feeds.append(Feed(iface.uri, arch = None, user_override = True))
 							writer.save_interface(interface)
 							d.destroy()
-							policy.recalculate()
+							import main
+							main.recalculate()
 				except zeroinstall.SafeException, ex:
 					error(str(ex))
 			else:
@@ -426,7 +430,8 @@ def add_local_feed(policy, interface):
 			writer.save_interface(interface)
 			chooser.destroy()
 			reader.update_from_cache(interface)
-			policy.recalculate()
+			import main
+			main.recalculate()
 		except Exception, ex:
 			dialog.alert(None, _("Error in feed file '%(feed)s':\n\n%(exception)s") % {'feed': feed, 'exception': str(ex)})
 
