@@ -27,6 +27,7 @@ def _get_feed_dir(feed):
 	if '#' in feed:
 		raise SafeException(_("Invalid URL '%s'") % feed)
 	scheme, rest = feed.split('://', 1)
+	assert '/' in rest, "Missing / in %s" % feed
 	domain, rest = rest.split('/', 1)
 	for x in [scheme, domain, rest]:
 		if not x or x.startswith(','):
@@ -153,6 +154,22 @@ class Fetcher(object):
 			return None
 		return '%s/%s/latest.xml' % (self.feed_mirror, _get_feed_dir(url))
 
+	@tasks.async
+	def get_packagekit_feed(self, iface_cache, feed_url):
+		"""Send a query to PackageKit (if available) for information about this package.
+		On success, the result is added to iface_cache.
+		"""
+		assert feed_url.startswith('distribution:'), feed_url
+		master_feed = iface_cache.get_feed(feed_url.split(':', 1)[1])
+		if master_feed:
+			fetch = iface_cache.distro.fetch_candidates(master_feed)
+			if fetch:
+				yield fetch
+				tasks.check(fetch)
+
+			# Force feed to be regenerated with the new information
+			iface_cache.get_feed(feed_url, force = True)
+
 	def download_and_import_feed(self, feed_url, iface_cache, force = False):
 		"""Download the feed, download any required keys, confirm trust if needed and import.
 		@param feed_url: the feed to be downloaded
@@ -163,7 +180,10 @@ class Fetcher(object):
 		from download import DownloadAborted
 		
 		debug(_("download_and_import_feed %(url)s (force = %(force)d)"), {'url': feed_url, 'force': force})
-		assert not feed_url.startswith('/')
+		assert not os.path.isabs(feed_url)
+
+		if feed_url.startswith('distribution:'):
+			return self.get_packagekit_feed(iface_cache, feed_url)
 
 		primary = self._download_and_import_feed(feed_url, iface_cache, force, use_mirror = False)
 
@@ -303,8 +323,7 @@ class Fetcher(object):
 		assert retrieval_method
 
 		if isinstance(retrieval_method, DistributionSource):
-			raise SafeException(_("This program depends on '%s', which is a package that is available through your distribution. "
-					"Please install it manually using your distribution's tools and try again.") % retrieval_method.package_id)
+			return retrieval_method.install(self.handler)
 
 		from zeroinstall.zerostore import manifest
 		best = None
@@ -415,8 +434,12 @@ class Fetcher(object):
 		return download_and_add_icon()
 
 	def download_impls(self, implementations, stores):
-		"""Download the given implementations, choosing a suitable retrieval method for each."""
-		blockers = []
+		"""Download the given implementations, choosing a suitable retrieval method for each.
+		If any of the retrieval methods are DistributionSources and
+		need confirmation, handler.confirm is called to check that the
+		installation should proceed.
+		"""
+		unsafe_impls = []
 
 		to_download = []
 		for impl in implementations:
@@ -428,14 +451,25 @@ class Fetcher(object):
 					"interface!)") % {'implementation_id': impl.id, 'interface': impl.feed.get_name()})
 			to_download.append((impl, source))
 
-		for impl, source in to_download:
-			blockers.append(self.download_impl(impl, source, stores))
-
-		if not blockers:
-			return None
+			if isinstance(source, DistributionSource) and source.needs_confirmation:
+				unsafe_impls.append(source.package_id)
 
 		@tasks.async
-		def download_impls(blockers):
+		def download_impls():
+			if unsafe_impls:
+				confirm = self.handler.confirm_install(_('The following components need to be installed using native packages. '
+					'These come from your distribution, and should therefore be trustworthy, but they also '
+					'run with extra privileges. In particular, installing them may run extra services on your '
+					'computer or affect other users. You may be asked to enter a password to confirm. The '
+					'packages are:\n\n') + ('\n'.join('- ' + x for x in unsafe_impls)))
+				yield confirm
+				tasks.check(confirm)
+
+			blockers = []
+
+			for impl, source in to_download:
+				blockers.append(self.download_impl(impl, source, stores))
+
 			# Record the first error log the rest
 			error = []
 			def dl_error(ex, tb = None):
@@ -451,7 +485,10 @@ class Fetcher(object):
 			if error:
 				raise error[0]
 
-		return download_impls(blockers)
+		if not to_download:
+			return None
+
+		return download_impls()
 
 	def get_best_source(self, impl):
 		"""Return the best download source for this implementation.
