@@ -174,16 +174,32 @@ class IfaceCache(object):
 	cache of L{model.Interface} objects, and an on-disk cache of L{model.ZeroInstallFeed}s.
 	It will probably be split into two in future.
 
+	@ivar distro: the native distribution proxy
+	@type distro: L{distro.Distribution}
+
 	@see: L{iface_cache} - the singleton IfaceCache instance.
 	"""
 
-	__slots__ = ['_interfaces', 'stores', '_feeds']
+	__slots__ = ['_interfaces', 'stores', '_feeds', '_distro']
 
-	def __init__(self):
+	def __init__(self, distro = None):
+		"""@param distro: distribution used to fetch "distribution:" feeds (since 0.49)
+		@param distro: distribution used to resolve "distribution:" feeds
+		@type distro: L{distro.Distribution}, or None to use the host distribution
+		"""
 		self._interfaces = {}
 		self._feeds = {}
 
 		self.stores = zerostore.Stores()
+
+		self._distro = distro
+
+	@property
+	def distro(self):
+		if self._distro is None:
+			from zeroinstall.injector.distro import get_host_distribution
+			self._distro = get_host_distribution()
+		return self._distro
 	
 	def update_interface_if_trusted(self, interface, sigs, xml):
 		import warnings
@@ -320,7 +336,16 @@ class IfaceCache(object):
 			if feed != False:
 				return feed
 
-		feed = self._feeds[url] = reader.load_feed_from_cache(url)
+		if url.startswith('distribution:'):
+			master_feed = self.get_feed(url.split(':', 1)[1])
+			if not master_feed:
+				return None	# Can't happen?
+			feed = self.distro.get_feed(master_feed)
+		else:
+			feed = reader.load_feed_from_cache(url)
+		if feed:
+			reader.update_user_feed_overrides(feed)
+		self._feeds[url] = feed
 		return feed
 
 	def get_interface(self, uri):
@@ -374,7 +399,7 @@ class IfaceCache(object):
 		@rtype: [L{gpg.Signature}] or None
 		@since: 0.25"""
 		import gpg
-		if uri.startswith('/'):
+		if os.path.isabs(uri):
 			old_iface = uri
 		else:
 			old_iface = basedir.load_first_cache(config_site, 'interfaces', escape(uri))
@@ -407,7 +432,7 @@ class IfaceCache(object):
 		If url is a local path, nothing happens.
 		This prevents us from repeatedly trying to download a failing feed many
 		times in a short period."""
-		if url.startswith('/'):
+		if os.path.isabs(url):
 			return
 		feeds_dir = basedir.save_cache_path(config_site, config_prog, 'last-check-attempt')
 		timestamp_path = os.path.join(feeds_dir, model._pretty_escape(url))
