@@ -14,7 +14,7 @@ well-known variables.
 # See the README file for details, or visit http://0install.net.
 
 from zeroinstall import _
-import os, re
+import os, re, locale
 from logging import info, debug, warn
 from zeroinstall import SafeException, version
 from zeroinstall.injector.namespaces import XMLNS_IFACE
@@ -68,6 +68,12 @@ def _join_arch(osys, machine):
 	if osys == machine == None: return None
 	return "%s-%s" % (osys or '*', machine or '*')
 
+def _best_language_match(options):
+	(language, encoding) = locale.getlocale(locale.LC_ALL)
+	return (options.get(language, None) or
+		options.get(language.split('_', 1)[0], None) or
+		options.get(None, None))
+
 class Stability(object):
 	"""A stability rating. Each implementation has an upstream stability rating and,
 	optionally, a user-set rating."""
@@ -237,9 +243,9 @@ class EnvironmentBinding(Binding):
 		if old_value is None:
 			return extra
 		if self.mode == EnvironmentBinding.PREPEND:
-			return extra + ':' + old_value
+			return extra + os.pathsep + old_value
 		else:
-			return old_value + ':' + extra
+			return old_value + os.pathsep + extra
 
 	def _toxml(self, doc):
 		"""Create a DOM element for this binding.
@@ -369,17 +375,23 @@ class Recipe(RetrievalMethod):
 
 class DistributionSource(RetrievalMethod):
 	"""A package that is installed using the distribution's tools (including PackageKit).
+	@ivar install: a function to call to install this package
+	@type install: (L{handler.Handler}) -> L{tasks.Blocker}
 	@ivar package_id: the package name, in a form recognised by the distribution's tools
 	@type package_id: str
 	@ivar size: the download size in bytes
-	@type size: int"""
+	@type size: int
+	@ivar needs_confirmation: whether the user should be asked to confirm before calling install()
+	@type needs_confirmation: bool"""
 
-	__slots__ = ['package_id', 'size']
+	__slots__ = ['package_id', 'size', 'install', 'needs_confirmation']
 
-	def __init__(self, package_id, size):
+	def __init__(self, package_id, size, install, needs_confirmation = True):
 		RetrievalMethod.__init__(self)
 		self.package_id = package_id
 		self.size = size
+		self.install = install
+		self.needs_confirmation = needs_confirmation
 
 class Implementation(object):
 	"""An Implementation is a package which implements an Interface.
@@ -467,20 +479,18 @@ class DistributionImplementation(Implementation):
 	"""An implementation provided by the distribution. Information such as the version
 	comes from the package manager.
 	@since: 0.28"""
+	__slots__ = ['distro', 'installed']
 
 	def __init__(self, feed, id, distro):
 		assert id.startswith('package:')
 		Implementation.__init__(self, feed, id)
 		self.distro = distro
+		self.installed = False
 
 	@property
 	def requires_root_install(self):
 		return not self.installed
 
-	@property
-	def installed(self):
-		return self.distro.get_installed(self.id)
-
 class ZeroInstallImplementation(Implementation):
 	"""An implementation where all the information comes from Zero Install.
 	@ivar digests: a list of "algorith=value" strings (since 0.45)
@@ -530,7 +540,7 @@ class Interface(object):
 
 	def __init__(self, uri):
 		assert uri
-		if uri.startswith('http:') or uri.startswith('https:') or uri.startswith('/'):
+		if uri.startswith('http:') or uri.startswith('https:') or os.path.isabs(uri):
 			self.uri = uri
 		else:
 			raise SafeException(_("Interface name '%s' doesn't start "
@@ -610,8 +620,10 @@ class ZeroInstallFeed(object):
 	@ivar implementations: Implementations in this feed, indexed by ID
 	@type implementations: {str: L{Implementation}}
 	@ivar name: human-friendly name
-	@ivar summary: short textual description
-	@ivar description: long textual description
+	@ivar summaries: short textual description (in various languages, since 0.49)
+	@type summaries: {str: str}
+	@ivar descriptions: long textual description (in various languages, since 0.49)
+	@type descriptions: {str: str}
 	@ivar last_modified: timestamp on signature
 	@ivar last_checked: time feed was last successfully downloaded and updated
 	@ivar feeds: list of <feed> elements in this feed
@@ -621,26 +633,31 @@ class ZeroInstallFeed(object):
 	@ivar metadata: extra elements we didn't understand
 	"""
 	# _main is deprecated
-	__slots__ = ['url', 'implementations', 'name', 'description', 'summary',
+	__slots__ = ['url', 'implementations', 'name', 'descriptions', 'summaries', '_package_implementations',
 		     'last_checked', 'last_modified', 'feeds', 'feed_for', 'metadata']
 
 	def __init__(self, feed_element, local_path = None, distro = None):
 		"""Create a feed object from a DOM.
 		@param feed_element: the root element of a feed file
 		@type feed_element: L{qdom.Element}
-		@param local_path: the pathname of this local feed, or None for remote feeds
-		@param distro: used to resolve distribution package references
-		@type distro: L{distro.Distribution} or None"""
-		assert feed_element
+		@param local_path: the pathname of this local feed, or None for remote feeds"""
 		self.implementations = {}
 		self.name = None
-		self.summary = None
-		self.description = ""
+		self.summaries = {}	# { lang: str }
+		self.descriptions = {}	# { lang: str }
 		self.last_modified = None
 		self.feeds = []
 		self.feed_for = set()
 		self.metadata = []
 		self.last_checked = None
+		self._package_implementations = []
+
+		if distro is not None:
+			import warnings
+			warnings.warn("distro argument is now ignored", DeprecationWarning, 2)
+
+		if feed_element is None:
+			return			# XXX subclass?
 
 		assert feed_element.name in ('interface', 'feed'), "Root element should be <interface>, not %s" % feed_element
 		assert feed_element.uri == XMLNS_IFACE, "Wrong namespace on root element: %s" % feed_element.uri
@@ -672,9 +689,9 @@ class ZeroInstallFeed(object):
 			if x.name == 'name':
 				self.name = x.content
 			elif x.name == 'description':
-				self.description = x.content
+				self.descriptions[x.attrs.get("http://www.w3.org/XML/1998/namespace lang", None)] = x.content
 			elif x.name == 'summary':
-				self.summary = x.content
+				self.summaries[x.attrs.get("http://www.w3.org/XML/1998/namespace lang", None)] = x.content
 			elif x.name == 'feed-for':
 				feed_iface = x.getAttribute('interface')
 				if not feed_iface:
@@ -700,8 +717,6 @@ class ZeroInstallFeed(object):
 		if not self.summary:
 			raise InvalidInterface(_("Missing <summary> in feed"))
 
-		package_impls = [0, []]		# Best score so far and packages with that score
-
 		def process_group(group, group_attrs, base_depends, base_bindings):
 			for item in group.childNodes:
 				if item.uri != XMLNS_IFACE: continue
@@ -733,14 +748,9 @@ class ZeroInstallFeed(object):
 				elif item.name == 'implementation':
 					process_impl(item, item_attrs, depends, bindings)
 				elif item.name == 'package-implementation':
-					distro_names = item_attrs.get('distributions', '')
-					for distro_name in distro_names.split(' '):
-						score = distro.get_score(distro_name)
-						if score > package_impls[0]:
-							package_impls[0] = score
-							package_impls[1] = []
-						if score == package_impls[0]:
-							package_impls[1].append((item, item_attrs, depends))
+					if depends:
+						warn("A <package-implementation> with dependencies in %s!", self.url)
+					self._package_implementations.append((item, item_attrs))
 				else:
 					assert 0
 
@@ -762,7 +772,7 @@ class ZeroInstallFeed(object):
 					# In older feeds, the ID was the (single) digest
 					impl.digests.append(id)
 			if id in self.implementations:
-				warn(_("Duplicate ID '%s' in feed '%s'"), id, self)
+				warn(_("Duplicate ID '%(id)s' in feed '%(feed)s'"), {'id': id, 'feed': self})
 			self.implementations[id] = impl
 
 			impl.metadata = item_attrs
@@ -840,39 +850,40 @@ class ZeroInstallFeed(object):
 					else:
 						impl.download_sources.append(recipe)
 
-		def process_native_impl(item, item_attrs, depends):
-			package = item_attrs.get('package', None)
-			if package is None:
-				raise InvalidInterface(_("Missing 'package' attribute on %s") % item)
-
-			def factory(id):
-				assert id.startswith('package:')
-				if id in self.implementations:
-					warn(_("Duplicate ID '%s' for DistributionImplementation"), id)
-				impl = DistributionImplementation(self, id, distro)
-				self.implementations[id] = impl
-
-				impl.metadata = item_attrs
-
-				item_main = item_attrs.get('main', None)
-				if item_main and not item_main.startswith('/'):
-					raise InvalidInterface(_("'main' attribute must be absolute, but '%s' doesn't start with '/'!") %
-								item_main)
-				impl.main = item_main
-				impl.upstream_stability = packaged
-				impl.requires = depends
-
-				return impl
-
-			distro.get_package_info(package, factory)
-		
 		root_attrs = {'stability': 'testing'}
 		if main:
 			root_attrs['main'] = main
 		process_group(feed_element, root_attrs, [], [])
+	
+	def get_distro_feed(self):
+		"""Does this feed contain any <pacakge-implementation> elements?
+		i.e. is it worth asking the package manager for more information?
+		@return: the URL of the virtual feed, or None
+		@since: 0.49"""
+		if self._package_implementations:
+			return "distribution:" + self.url
+		return None
 
-		for args in package_impls[1]:
-			process_native_impl(*args)
+	def get_package_impls(self, distro):
+		"""Find the best <pacakge-implementation> element(s) for the given distribution.
+		@param distro: the distribution to use to rate them
+		@type distro: L{distro.Distribution}
+		@return: a list of tuples for the best ranked elements
+		@rtype: [str]
+		@since: 0.49"""
+		best_score = 0
+		best_impls = []
+
+		for item, item_attrs in self._package_implementations:
+			distro_names = item_attrs.get('distributions', '')
+			for distro_name in distro_names.split(' '):
+				score = distro.get_score(distro_name)
+				if score > best_score:
+					best_score = score
+					best_impls = []
+				if score == best_score:
+					best_impls.append((item, item_attrs))
+		return best_impls
 
 	def get_name(self):
 		return self.name or '(' + os.path.basename(self.url) + ')'
@@ -912,6 +923,14 @@ class ZeroInstallFeed(object):
 		"""Return a list of interface metadata elements with this name and namespace URI."""
 		return [m for m in self.metadata if m.name == name and m.uri == uri]
 
+	@property
+	def summary(self):
+		return _best_language_match(self.summaries)
+
+	@property
+	def description(self):
+		return _best_language_match(self.descriptions)
+
 class DummyFeed(object):
 	"""Temporary class used during API transition."""
 	last_modified = None
