@@ -142,6 +142,9 @@ class Policy(object):
 	
 	def recalculate(self, fetch_stale_interfaces = True):
 		"""@deprecated: see L{solve_with_downloads} """
+		import warnings
+		warnings.warn("Policy.recalculate is deprecated!", DeprecationWarning, stacklevel = 2)
+
 		self.stale_feeds = set()
 
 		host_arch = self.target_arch
@@ -154,7 +157,7 @@ class Policy(object):
 
 		blockers = []
 		for f in self.solver.feeds_used:
-			if f.startswith('/'): continue
+			if os.path.isabs(f): continue
 			feed = iface_cache.get_feed(f)
 			if feed is None or feed.last_modified is None:
 				self.download_and_import_feed_if_online(f)	# Will start a download
@@ -191,7 +194,7 @@ class Policy(object):
 		@return: true if feed is stale or missing."""
 		if feed is None:
 			return True
-		if feed.url.startswith('/'):
+		if os.path.isabs(feed.url):
 			return False		# Local feeds are never stale
 		if feed.last_modified is None:
 			return True		# Don't even have it yet
@@ -295,11 +298,12 @@ class Policy(object):
 		return [iface_cache.get_interface(uri) for uri in feed_targets]
 	
 	@tasks.async
-	def solve_with_downloads(self, force = False):
+	def solve_with_downloads(self, force = False, update_local = False):
 		"""Run the solver, then download any feeds that are missing or
 		that need to be updated. Each time a new feed is imported into
 		the cache, the solver is run again, possibly adding new downloads.
-		@param force: whether to download even if we're already ready to run."""
+		@param force: whether to download even if we're already ready to run.
+		@param update_local: fetch PackageKit feeds even if we're ready to run."""
 		
 		downloads_finished = set()		# Successful or otherwise
 		downloads_in_progress = {}		# URL -> Download
@@ -308,39 +312,65 @@ class Policy(object):
 		if self.src:
 			host_arch = arch.SourceArchitecture(host_arch)
 
+		# There are three cases:
+		# 1. We want to run immediately if possible. If not, download all the information we can.
+		#    (force = False, update_local = False)
+		# 2. We're in no hurry, but don't want to use the network unnecessarily.
+		#    We should still update local information (from PackageKit).
+		#    (force = False, update_local = True)
+		# 3. The user explicitly asked us to refresh everything.
+		#    (force = True)
+
+		try_quick_exit = not (force or update_local)
+
 		while True:
 			self.solver.solve(self.root, host_arch)
 			for w in self.watchers: w()
 
-			if self.solver.ready and not force:
+			if try_quick_exit and self.solver.ready:
 				break
-			else:
-				if self.network_use == network_offline and not force:
-					info(_("Can't choose versions and in off-line mode, so aborting"))
-					break
-				# Once we've starting downloading some things,
-				# we might as well get them all.
+			try_quick_exit = False
+
+			if not self.solver.ready:
 				force = True
 
 			for f in self.solver.feeds_used:
 				if f in downloads_finished or f in downloads_in_progress:
 					continue
-				if f.startswith('/'):
+				if os.path.isabs(f):
 					continue
-				downloads_in_progress[f] = self.fetcher.download_and_import_feed(f, iface_cache)
+				elif f.startswith('distribution:'):
+					if force or update_local:
+						downloads_in_progress[f] = self.fetcher.download_and_import_feed(f, iface_cache)
+				elif force and self.network_use != network_offline:
+					downloads_in_progress[f] = self.fetcher.download_and_import_feed(f, iface_cache)
+					# Once we've starting downloading some things,
+					# we might as well get them all.
+					force = True
 
 			if not downloads_in_progress:
+				if self.network_use == network_offline:
+					info(_("Can't choose versions and in off-line mode, so aborting"))
 				break
 
+			# Wait for at least one download to finish
 			blockers = downloads_in_progress.values()
 			yield blockers
 			tasks.check(blockers, self.handler.report_error)
 
 			for f in downloads_in_progress.keys():
-				if downloads_in_progress[f].happened:
+				if f in downloads_in_progress and downloads_in_progress[f].happened:
 					del downloads_in_progress[f]
 					downloads_finished.add(f)
 
+					# Need to refetch any "distribution" feed that
+					# depends on this one
+					distro_feed_url = 'distribution:' + f
+					if distro_feed_url in downloads_finished:
+						downloads_finished.remove(distro_feed_url)
+					if distro_feed_url in downloads_in_progress:
+						del downloads_in_progress[distro_feed_url]
+
 	@tasks.async
 	def solve_and_download_impls(self, refresh = False, select_only = False):
 		"""Run L{solve_with_downloads} and then get the selected implementations too.
