@@ -231,7 +231,18 @@ class Stores(object):
 		if impl_dirs:
 			dirs = file(impl_dirs)
 		else:
-			dirs = ['/var/cache/0install.net/implementations']
+			if os.name == "nt":
+				from win32com.shell import shell, shellcon
+				localAppData = shell.SHGetFolderPath(0, shellcon.CSIDL_LOCAL_APPDATA, 0, 0)
+				commonAppData = shell.SHGetFolderPath(0, shellcon.CSIDL_COMMON_APPDATA, 0, 0)
+
+				userCache = os.path.join(localAppData, "0install.net", "implementations")
+				sharedCache = os.path.join(commonAppData, "0install.net", "implementations")
+				dirs = [userCache, sharedCache]
+
+			else:
+				dirs = ['/var/cache/0install.net/implementations']
+
 		for directory in dirs:
 			directory = directory.strip()
 			if directory and not directory.startswith('#'):
